==> verum-dezyne-2.16.0-x86_64-windows.zip: dezyne-2.16.0/runtime/c++/runtime.cc <==
// dzn-runtime -- Dezyne runtime library
//
// Copyright © 2014, 2015, 2016, 2017, 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
// Copyright © 2015, 2016, 2017, 2019, 2020, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
// Copyright © 2015 Paul Hoogendijk <paul@dezyne.org>
//
// This file is part of dzn-runtime.
//
// dzn-runtime is free software: you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation, either version 3 of the
// License, or (at your option) any later version.
//
// dzn-runtime is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with dzn-runtime.  If not, see <http://www.gnu.org/licenses/>.
//
// Commentary:
//
// Code:

#include <dzn/runtime.hh>
#include <dzn/coroutine.hh>

#include <algorithm>
#include <iostream>

namespace dzn
{
  std::ostream debug(nullptr);
  runtime::runtime(){}

  void trace(std::ostream& os, port::meta const& meta, const char* event_name)
  {
    if (!os.rdbuf())
      return;
    os << path(meta.require.meta, meta.require.name) << "." << event_name << " -> "
       << path(meta.provide.meta, meta.provide.name) << "." << event_name << std::endl;
  }

  void trace_out(std::ostream& os, port::meta const& meta, const char* event_name)
  {
    if (!os.rdbuf())
      return;
    os << path(meta.require.meta, meta.require.name) << "." << event_name << " <- "
       << path(meta.provide.meta, meta.provide.name) << "." << event_name << std::endl;
  }

  void trace_qin(std::ostream& os, port::meta const& meta, const char* event_name)
  {
    if (!os.rdbuf())
      return;
    if (path (meta.require.meta, "") == "<external>")
      trace_out (os, meta, event_name);
    else
      os <<  path (meta.require.meta, "<q>")
         << " <- "
         << path (meta.provide.meta, meta.provide.name) << "." << event_name << std::endl;
  }

  void trace_qout(std::ostream& os, port::meta const& meta, const char* event_name)
  {
    if (!os.rdbuf())
      return;
    if (path (meta.require.meta, "") == "<external>")
      return;
    os << path (meta.require.meta, meta.require.name) << "." << event_name
       << " <- "
       << path(meta.require.meta, "<q>") << std::endl;
  }

  bool runtime::external(void* scope) {
    return (states.find(scope) == states.end());
  }

  size_t& runtime::handling(void* scope)
  {
    return states[scope].handling;
  }

  size_t& runtime::blocked(void* scope)
  {
    return states[scope].blocked;
  }

  void*& runtime::deferred(void* scope)
  {
    return states[scope].deferred;
  }

  std::queue<std::function<void()> >& runtime::queue(void* scope)
  {
    return states[scope].queue;
  }

  bool& runtime::performs_flush(void* scope)
  {
    return states[scope].performs_flush;
  }

  bool& runtime::skip_block(void* port)
  {
    return skip_port[port];
  }

  void runtime::flush(void* scope, size_t coroutine_id)
  {
    handling(scope) = 0;
    if(!external(scope))
    {
      std::queue<std::function<void()> >& q = queue(scope);
      while(! q.empty())
      {
        std::function<void()> event = q.front();
        q.pop();
        handle(scope, event, coroutine_id);
        handling(scope) = 0;
      }
      if (deferred(scope)) {
        void* tgt = deferred(scope);
        deferred(scope) = nullptr;
        if (!handling(tgt)) {
          runtime::flush(tgt, coroutine_id);
        }
      }
    }
  }

  void runtime::enqueue(void* source, void* target, const std::function<void()>& event, size_t coroutine_id)
  {
    if(!(source && performs_flush(source)) && !handling(target))
    {
      handle(target, event, coroutine_id);
      flush(target, coroutine_id);
    }
    else
    {
      deferred(source) = target;
      queue(target).push(event);
    }
  }
}
//version: 2.16.0

==> verum-dezyne-2.16.1-x86_64-windows.zip: dezyne-2.16.1/runtime/c++/runtime.cc <==
// dzn-runtime -- Dezyne runtime library
//
// Copyright © 2014, 2015, 2016, 2017, 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
// Copyright © 2015, 2016, 2017, 2019, 2020, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
// Copyright © 2015 Paul Hoogendijk <paul@dezyne.org>
//
// This file is part of dzn-runtime.
//
// dzn-runtime is free software: you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation, either version 3 of the
// License, or (at your option) any later version.
//
// dzn-runtime is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with dzn-runtime.  If not, see <http://www.gnu.org/licenses/>.
//
// Commentary:
//
// Code:

#include <dzn/runtime.hh>
#include <dzn/coroutine.hh>

#include <algorithm>
#include <iostream>

namespace dzn
{
  std::ostream debug(nullptr);
  runtime::runtime(){}

  void trace(std::ostream& os, port::meta const& meta, const char* event_name)
  {
    if (!os.rdbuf())
      return;
    os << path(meta.require.meta, meta.require.name) << "." << event_name << " -> "
       << path(meta.provide.meta, meta.provide.name) << "." << event_name << std::endl;
  }

  void trace_out(std::ostream& os, port::meta const& meta, const char* event_name)
  {
    if (!os.rdbuf())
      return;
    os << path(meta.require.meta, meta.require.name) << "." << event_name << " <- "
       << path(meta.provide.meta, meta.provide.name) << "." << event_name << std::endl;
  }

  void trace_qin(std::ostream& os, port::meta const& meta, const char* event_name)
  {
    if (!os.rdbuf())
      return;
    if (path (meta.require.meta, "") == "<external>")
      trace_out (os, meta, event_name);
    else
      os <<  path (meta.require.meta, "<q>")
         << " <- "
         << path (meta.provide.meta, meta.provide.name) << "." << event_name << std::endl;
  }

  void trace_qout(std::ostream& os, port::meta const& meta, const char* event_name)
  {
    if (!os.rdbuf())
      return;
    if (path (meta.require.meta, "") == "<external>")
      return;
    os << path (meta.require.meta, meta.require.name) << "." << event_name
       << " <- "
       << path(meta.require.meta, "<q>") << std::endl;
  }

  bool runtime::external(void* component) {
    return (states.find(component) == states.end());
  }

  size_t& runtime::handling(void* component)
  {
    return states[component].handling;
  }

  size_t& runtime::blocked(void* component)
  {
    return states[component].blocked;
  }

  void*& runtime::deferred(void* component)
  {
    return states[component].deferred;
  }

  std::queue<std::function<void()> >& runtime::queue(void* component)
  {
    return states[component].queue;
  }

  bool& runtime::performs_flush(void* component)
  {
    return states[component].performs_flush;
  }

  bool runtime::skip_block(void* component, void* port)
  {
    return states[component].skip == port;
  }

  void runtime::set_skip_block(void* component, void* port)
  {
    states[component].skip = port;
  }

  void runtime::reset_skip_block(void* component)
  {
    states[component].skip = nullptr;
  }

  void runtime::flush(void* component, size_t coroutine_id)
  {
    handling(component) = 0;
    if(!external(component))
    {
      std::queue<std::function<void()> >& q = queue(component);
      while(! q.empty())
      {
        std::function<void()> event = q.front();
        q.pop();
        handle(component, event, coroutine_id);
        handling(component) = 0;
      }
      if (deferred(component)) {
        void* tgt = deferred(component);
        deferred(component) = nullptr;
        if (!handling(tgt)) {
          runtime::flush(tgt, coroutine_id);
        }
      }
    }
  }

  void runtime::enqueue(void* source, void* target, const std::function<void()>& event, size_t coroutine_id)
  {
    if(!(source && performs_flush(source)) && !handling(target))
    {
      handle(target, event, coroutine_id);
      flush(target, coroutine_id);
    }
    else
    {
      deferred(source) = target;
      queue(target).push(event);
    }
  }
}
//version: 2.16.1
